@@ -21,7 +21,7 @@ __all__ = ['CompiledSimulation']
 
 
 class DllMemInspector(collections.Mapping):
-    """Dictionary-like access to a hashmap in a CompiledSimulation."""
+    """ Dictionary-like access to a hashmap in a CompiledSimulation. """
 
     def __init__(self, sim, mem):
         self._aw = mem.addrwidth
@@ -86,10 +86,22 @@ class CompiledSimulation(object):
         self._remove_untraceable()
 
         self.default_value = default_value
-        self._regmap, self._memmap = register_value_map, memory_value_map
+        self._regmap = {}  # Updated below
+        self._memmap = memory_value_map
         self._uid_counter = 0
         self.varname = {}  # mapping from wires and memories to C variables
 
+        for r in self.block.wirevector_subset(Register):
+            rval = register_value_map.get(r, r.reset_value)
+            if rval is None:
+                rval = self.default_value
+            self._regmap[r] = rval
+
+        # Passing the dictionary objects themselves since they aren't updated anywhere.
+        # If that's ever not the case, will need to pass in deep copies of them like done
+        # for the normal Simulation so we retain the initial values that had.
+        self.tracer._set_initial_values(default_value, self._regmap, self._memmap)
+
         self._create_dll()
         self._initialize_mems()
 
@@ -125,7 +137,7 @@ class CompiledSimulation(object):
 
         :param provided_inputs: a dictionary mapping wirevectors to their values for N steps
         :param expected_outputs: a dictionary mapping wirevectors to their expected values
-            for N steps
+            for N steps; use '?' to indicate you don't care what the value at that step is
         :param nsteps: number of steps to take (defaults to None, meaning step for each
             supplied input value)
         :param file: where to write the output (if there are unexpected outputs detected)
@@ -202,7 +214,10 @@ class CompiledSimulation(object):
             self.step({w: int(v[i]) for w, v in provided_inputs.items()})
 
             for expvar in expected_outputs.keys():
-                expected = int(expected_outputs[expvar][i])
+                expected = expected_outputs[expvar][i]
+                if expected == '?':
+                    continue
+                expected = int(expected)
                 actual = self.inspect(expvar)
                 if expected != actual:
                     failed.append((i, expvar, expected, actual))
@@ -229,10 +244,10 @@ class CompiledSimulation(object):
             file.flush()
 
     def run(self, inputs):
-        """Run many steps of the simulation.
+        """ Run many steps of the simulation.
 
         :param inputs: A list of input mappings for each step;
-          its length is the number of steps to be executed.
+            its length is the number of steps to be executed.
         """
         steps = len(inputs)
         # create i/o arrays of the appropriate length
@@ -288,7 +303,7 @@ class CompiledSimulation(object):
             self.tracer.trace[name].extend(res)
 
     def _traceable(self, wv):
-        """Check if wv is able to be traced
+        """ Check if wv is able to be traced.
 
         If it is traceable due to a probe, record that probe in _probe_mapping.
         """
@@ -301,7 +316,7 @@ class CompiledSimulation(object):
         return False
 
     def _remove_untraceable(self):
-        """Remove from the tracer those wires that CompiledSimulation cannot track.
+        """ Remove from the tracer those wires that CompiledSimulation cannot track.
 
         Create _probe_mapping for wires only traceable via probes.
         """
@@ -312,7 +327,7 @@ class CompiledSimulation(object):
         self.tracer.trace.__init__(wvs)
 
     def _create_dll(self):
-        """Create a dynamically-linked library implementing the simulation logic."""
+        """ Create a dynamically-linked library implementing the simulation logic. """
         self._dir = tempfile.mkdtemp()
         with open(path.join(self._dir, 'pyrtlsim.c'), 'w') as f:
             self._create_code(lambda s: f.write(s + '\n'))
@@ -334,11 +349,11 @@ class CompiledSimulation(object):
         self._mem_lookup.restype = ctypes.POINTER(ctypes.c_uint64)
 
     def _limbs(self, w):
-        """Number of 64-bit words needed to store value of wire."""
+        """ Number of 64-bit words needed to store value of wire. """
         return (w.bitwidth + 63) // 64
 
     def _makeini(self, w, v):
-        """C initializer string for a wire with a given value."""
+        """ C initializer string for a wire with a given value. """
         pieces = []
         for _ in range(self._limbs(w)):
             pieces.append(hex(v & ((1 << 64) - 1)))
@@ -346,7 +361,7 @@ class CompiledSimulation(object):
         return ','.join(pieces).join('{}')
 
     def _romwidth(self, m):
-        """Bitwidth of integer type sufficient to hold rom entry.
+        """ Bitwidth of integer type sufficient to hold rom entry.
 
         On large memories, returns 64; an array will be needed.
         """
@@ -369,18 +384,18 @@ class CompiledSimulation(object):
         return ''
 
     def _getarglimb(self, arg, n):
-        """Get the nth limb of the given wire.
+        """ Get the nth limb of the given wire.
 
         Returns '0' when the wire does not have sufficient limbs.
         """
         return '{vn}[{n}]'.format(vn=self.varname[arg], n=n) if arg.bitwidth > 64 * n else '0'
 
     def _clean_name(self, prefix, obj):
-        """Create a C variable name with the given prefix based on the name of obj."""
+        """ Create a C variable name with the given prefix based on the name of obj. """
         return '{}{}_{}'.format(prefix, self._uid(), ''.join(c for c in obj.name if c.isalnum()))
 
     def _uid(self):
-        """Get an auto-incrementing number suitable for use as a unique identifier."""
+        """ Get an auto-incrementing number suitable for use as a unique identifier. """
         x = self._uid_counter
         self._uid_counter += 1
         return x
@@ -427,9 +442,11 @@ class CompiledSimulation(object):
             write('const uint64_t {name}[{limbs}] = {val};'.format(
                 limbs=self._limbs(w), name=vn, val=self._makeini(w, w.val)))
         elif isinstance(w, Register):
+            rval = self._regmap.get(w, w.reset_value)
+            if rval is None:
+                rval = self.default_value
             write('static uint64_t {name}[{limbs}] = {val};'.format(
-                limbs=self._limbs(w), name=vn,
-                val=self._makeini(w, self._regmap.get(w, self.default_value))))
+                limbs=self._limbs(w), name=vn, val=self._makeini(w, rval)))
         else:
             write('uint64_t {name}[{limbs}];'.format(limbs=self._limbs(w), name=vn))
 
@@ -705,7 +722,7 @@ class CompiledSimulation(object):
         self._declare_mem_helpers(write)
         roms = {mem for mem in mems if isinstance(mem, RomBlock)}
         self._declare_roms(write, roms)
-        mems = {mem for mem in mems if isinstance(mem, MemBlock)}
+        mems = {mem for mem in mems if isinstance(mem, MemBlock) and not isinstance(mem, RomBlock)}
         self._declare_mems(write, mems)
 
         # single step function
