@@ -279,7 +279,7 @@ class Block(object):
         self.wirevector_by_name[wirevector.name] = wirevector
 
     def remove_wirevector(self, wirevector):
-        """ Remove a wirevector object to the block."""
+        """ Remove a wirevector object from the block."""
         self.wirevector_set.remove(wirevector)
         del self.wirevector_by_name[wirevector.name]
 
@@ -370,7 +370,8 @@ class Block(object):
         If no cls is specified, the full set of wirevectors associated with the Block are
         returned.  If cls is a single type, or a tuple of types, only those wirevectors of
         the matching types will be returned.  This is helpful for getting all inputs, outputs,
-        or registers of a block for example."""
+        or registers of a block for example.
+        """
         if cls is None:
             initial_set = self.wirevector_set
         else:
@@ -384,7 +385,8 @@ class Block(object):
         """Return set of logicnets, filtered by the type(s) of logic op provided as op.
 
         If no op is specified, the full set of logicnets associated with the Block are
-        returned.  This is helpful for getting all memories of a block for example."""
+        returned.  This is helpful for getting all memories of a block for example.
+        """
         if op is None:
             return self.logic
         else:
@@ -395,7 +397,8 @@ class Block(object):
 
         By fallthrough, if a matching wirevector cannot be found the value None is
         returned.  However, if the argument strict is set to True, then this will
-        instead throw a PyrtlError when no match is found."""
+        instead throw a PyrtlError when no match is found.
+        """
         if name in self.wirevector_by_name:
             return self.wirevector_by_name[name]
         elif strict:
@@ -457,7 +460,7 @@ class Block(object):
 
     def _repr_svg_(self):
         """ IPython display support for Block. """
-        from .inputoutput import block_to_svg
+        from .visualization import block_to_svg
         return block_to_svg(self)
 
     def __iter__(self):
@@ -468,7 +471,8 @@ class Block(object):
         Note: this method will throw an error if there are loops in the
         logic that do not involve registers.
         Also, the order of the nets is not guaranteed to be the same
-        over multiple iterations"""
+        over multiple iterations.
+        """
         from .wire import Input, Const, Register
         src_dict, dest_dict = self.net_connections()
         to_clear = self.wirevector_subset((Input, Const, Register))
@@ -498,9 +502,9 @@ class Block(object):
         """ Check block and throw PyrtlError or PyrtlInternalError if there is an issue.
 
         Should not modify anything, only check data structures to make sure they have been
-        built according to the assumptions stated in the Block comments."""
+        built according to the assumptions stated in the Block comments.
+        """
 
-        # TODO: check that the wirevector_by_name is sane
         from .wire import Input, Const, Output
         from .helperfuncs import get_stack, get_stacks
 
@@ -524,9 +528,6 @@ class Block(object):
                              'or "const_" as a signal name because those are reserved for '
                              'internal use)' % repr(wirevector_names_list))
 
-        # check for dead input wires (not connected to anything)
-        all_input_and_consts = self.wirevector_subset((Input, Const))
-
         # The following line also checks for duplicate wire drivers
         wire_src_dict, wire_dst_dict = self.net_connections()
         dest_set = set(wire_src_dict.keys())
@@ -537,9 +538,12 @@ class Block(object):
             bad_wire_names = '\n    '.join(str(x) for x in connected_minus_allwires)
             raise PyrtlError('Unknown wires found in net:\n %s \n\n %s' % (bad_wire_names,
                              get_stacks(*connected_minus_allwires)))
+
+        all_input_and_consts = self.wirevector_subset((Input, Const))
+
+        # Check for wires that aren't connected to anything (inputs and consts can be unconnected)
         allwires_minus_connected = self.wirevector_set.difference(full_set)
         allwires_minus_connected = allwires_minus_connected.difference(all_input_and_consts)
-        #   ^ allow inputs and consts to be unconnected
         if len(allwires_minus_connected) > 0:
             bad_wire_names = '\n    '.join(str(x) for x in allwires_minus_connected)
             raise PyrtlError('Wires declared but not connected:\n %s \n\n %s' % (bad_wire_names,
@@ -556,6 +560,24 @@ class Block(object):
         # Check for async memories not specified as such
         self.sanity_check_memory_sync(wire_src_dict)
 
+        # Check that all mappings in wirevector_by_name are consistent
+        bad_wv_by_name = [w for n, w in self.wirevector_by_name.items() if n != w.name]
+        if bad_wv_by_name:
+            raise PyrtlInternalError('Wires with inconsistent entry in wirevector_by_name '
+                                     'dict: %s' % [w.name for w in bad_wv_by_name])
+
+        # Check that all wires are in wirevector_by_name
+        wv_by_name_set = set(self.wirevector_by_name.keys())
+        missing_wires = wirevector_names_set.difference(wv_by_name_set)
+        if missing_wires:
+            raise PyrtlInternalError('Missing entries in wirevector_by_name for the '
+                                     'following wires: %s' % missing_wires)
+
+        unknown_wires = wv_by_name_set.difference(wirevector_names_set)
+        if unknown_wires:
+            raise PyrtlInternalError('Unknown wires found in wirevector_by_name: %s'
+                                     % unknown_wires)
+
         if debug_mode:
             # Check for wires that are destinations of a logicNet, but are not outputs and are never
             # used as args.
@@ -612,16 +634,16 @@ class Block(object):
 
     def sanity_check_memblock(self, m):
         """ Check that m is a valid memblock type. """
-        from .memory import _MemReadBase
-        if not isinstance(m, _MemReadBase):
+        from .memory import MemBlock
+        if not isinstance(m, MemBlock):
             raise PyrtlError(
                 'error attempting to pass an input of type "%s" '
-                'instead of _MemReadBase' % type(m))
+                'instead of MemBlock' % type(m))
 
     def sanity_check_net(self, net):
         """ Check that net is a valid LogicNet. """
         from .wire import Input, Output, Const, Register
-        from .memory import _MemReadBase
+        from .memory import MemBlock
 
         # general sanity checks that apply to all operations
         if not isinstance(net, LogicNet):
@@ -692,7 +714,7 @@ class Block(object):
                 raise PyrtlInternalError('error, mem op requires 2 op_params in tuple')
             if not isinstance(net.op_param[0], int):
                 raise PyrtlInternalError('error, mem op requires first operand as int')
-            if not isinstance(net.op_param[1], _MemReadBase):
+            if not isinstance(net.op_param[1], MemBlock):
                 raise PyrtlInternalError('error, mem op requires second operand of a memory type')
 
         # operation-specific checks on destinations
@@ -724,12 +746,22 @@ class Block(object):
 
 class PostSynthBlock(Block):
     """ This is a block with extra metadata required to maintain the
-    pre synthesis interface post synthesis
+    pre-synthesis interface during post-synthesis.
+
+    It currently holds the following instance attributes:
+
+    * *.io_map*: a map from old IO wirevector to a list of new IO wirevectors it maps to;
+        this is a list because for unmerged io vectors, each old N-bit IO wirevector maps
+        to N new 1-bit IO wirevectors.
+    * *.reg_map*: a map from old register to a list of new registers; a list because post-synthesis,
+        each N-bit register has been mapped to N 1-bit registers
+    * *.mem_map*: a map from old memory block to the new memory block
     """
 
     def __init__(self):
         super(PostSynthBlock, self).__init__()
-        self.io_map = {}
+        self.io_map = collections.defaultdict(list)
+        self.reg_map = collections.defaultdict(list)
         self.mem_map = {}
 
 
@@ -758,7 +790,8 @@ def _get_debug_mode():
 
 
 def _get_useful_callpoint_name():
-    """ Attempts to find the lowest user-level call into the pyrtl module
+    """ Attempts to find the lowest user-level call into the PyRTL module.
+
     :return (string, int) or None: the file name and line number respectively
 
     This function walks back the current frame stack attempting to find the
@@ -815,10 +848,10 @@ def reset_working_block():
 
 class set_working_block(object):
     """ Set the working block to be the block passed as argument.
-        Compatible with the 'with' statement
+    Compatible with the 'with' statement.
 
-        Sanity checks will only be run if the new block is different
-        from the original block
+    Sanity checks will only be run if the new block is different
+    from the original block.
     """
 
     @staticmethod
@@ -844,14 +877,24 @@ class set_working_block(object):
 
 def temp_working_block():
     """ Set the working block to be new temporary block.
-        If used with the 'with' statement the block will be reset to the
-        original value (at the time of call) at exit of the context.
+
+    If used with the 'with' statement the block will be reset to the
+    original value (at the time of call) at exit of the context.
     """
     return set_working_block(Block())
 
 
 def set_debug_mode(debug=True):
-    """ Set the global debug mode. """
+    """ Set the global debug mode.
+
+    :param debug: Optional boolean paramter to which debug mode will be set
+
+    This function will set the debug mode to the specified value.  Debug mode
+    is, by default, set to off to keep the performance of the system.  With debug
+    mode set to true, all temporary WireVectors created will be given a name based
+    on the line of code on which they were created and a snapshot of the call-stack
+    for those WireVectors will be kept as well.
+    """
     global debug_mode
     global _setting_keep_wirevector_call_stack
     global _setting_slower_but_more_descriptive_tmps
@@ -864,13 +907,13 @@ _py_regex = r'^[^\d\W]\w*\Z'
 
 
 class _NameIndexer(object):
-    """ Provides internal names that are based on a prefix and an index"""
+    """ Provides internal names that are based on a prefix and an index. """
     def __init__(self, internal_prefix='_sani_temp'):
         self.internal_prefix = internal_prefix
         self.internal_index = 0
 
     def make_valid_string(self):
-        """Build a valid string based on the prefix and internal index"""
+        """ Build a valid string based on the prefix and internal index. """
         return self.internal_prefix + str(self.next_index())
 
     def next_index(self):
@@ -880,9 +923,8 @@ class _NameIndexer(object):
 
 
 class _NameSanitizer(_NameIndexer):
-    """
-    Sanitizes the names so that names can be used in places that don't allow
-    for arbitrary names while not mangling valid names
+    """ Sanitizes the names so that names can be used in places that don't allow
+    for arbitrary names while not mangling valid names.
 
     Put the values you want to validate into make_valid_string the first time
     you want to sanitize a particular string (or before the first time), and
@@ -902,7 +944,7 @@ class _NameSanitizer(_NameIndexer):
         super(_NameSanitizer, self).__init__(internal_prefix)
 
     def __getitem__(self, item):
-        """ Get a value from the sanitizer"""
+        """ Get a value from the sanitizer """
         if not self.map_valid and self.is_valid_str(item):
             return item
         return self.val_map[item]
@@ -911,7 +953,7 @@ class _NameSanitizer(_NameIndexer):
         return self.identifier.match(string) and self.extra_checks(string)
 
     def make_valid_string(self, string=''):
-        """ Inputting a value for the first time """
+        """ Inputting a value for the first time. """
         if not self.is_valid_str(string):
             if string in self.val_map and not self.allow_dups:
                 raise IndexError("Value {} has already been given to the sanitizer".format(string))
@@ -925,7 +967,7 @@ class _NameSanitizer(_NameIndexer):
 
 
 class _PythonSanitizer(_NameSanitizer):
-    """ Name Sanitizer specifically built for Python identifers"""
+    """ Name Sanitizer specifically built for Python identifers. """
     def __init__(self, internal_prefix='_sani_temp', map_valid_vals=True):
         super(_PythonSanitizer, self).__init__(_py_regex, internal_prefix, map_valid_vals)
         self.extra_checks = lambda s: not keyword.iskeyword(s)
