@@ -5,6 +5,7 @@ ways to change a block.
 """
 
 from __future__ import print_function, unicode_literals
+import collections
 
 from .core import working_block, set_working_block, _get_debug_mode, LogicNet, PostSynthBlock
 from .helperfuncs import _NetCount
@@ -15,7 +16,8 @@ from .memory import MemBlock
 from .pyrtlexceptions import PyrtlError, PyrtlInternalError
 from .wire import WireVector, Input, Output, Const, Register
 from .transform import net_transform, _get_new_block_mem_instance, copy_block, replace_wires
-from . import transform  # transform.all_nets looks better than all_nets
+from . import transform
+from pyrtl import wire  # transform.all_nets looks better than all_nets
 
 
 # --------------------------------------------------------------------
@@ -29,9 +31,12 @@ def optimize(update_working_block=True, block=None, skip_sanity_check=False):
     """
     Return an optimized version of a synthesized hardware block.
 
-    :param Boolean update_working_block: Don't copy the block and optimize the
-    new block
+    :param bool update_working_block: Don't copy the block and optimize the
+        new block (defaults to True)
     :param Block block: the block to optimize (defaults to working block)
+    :param bool skip_sanity_check: Don't perform sanity checks on the block
+        before/during/after the optimization passes (defaults to False).
+        Sanity checks will always be performed if in debug mode.
 
     Note:
     optimize works on all hardware designs, both synthesized and non synthesized
@@ -43,7 +48,8 @@ def optimize(update_working_block=True, block=None, skip_sanity_check=False):
     with set_working_block(block, no_sanity_check=True):
         if (not skip_sanity_check) or _get_debug_mode():
             block.sanity_check()
-        _remove_wire_nets(block)
+        _remove_wire_nets(block, skip_sanity_check)
+        _remove_slice_nets(block, skip_sanity_check)
         constant_propagation(block, True)
         _remove_unlistened_nets(block)
         common_subexp_elimination(block)
@@ -53,8 +59,7 @@ def optimize(update_working_block=True, block=None, skip_sanity_check=False):
 
 
 class _ProducerList(object):
-    """  Maps from wire to its immediate producer and finds ultimate producers
-    """
+    """  Maps from wire to its immediate producer and finds ultimate producers. """
     def __init__(self):
         self.dict = {}  # map from wirevector to its direct producer wirevector
 
@@ -71,7 +76,7 @@ class _ProducerList(object):
             return item
 
 
-def _remove_wire_nets(block):
+def _remove_wire_nets(block, skip_sanity_check=False):
     """ Remove all wire nodes from the block. """
 
     wire_src_dict = _ProducerList()
@@ -93,13 +98,75 @@ def _remove_wire_nets(block):
             new_net = LogicNet(net.op, net.op_param, new_args, net.dests)
             new_logic.add(new_net)
 
+    # now update the block with the new logic and remove wirevectors
+    block.logic = new_logic
+    for dead_wirevector in wire_removal_set:
+        block.remove_wirevector(dead_wirevector)
+
+    if (not skip_sanity_check) or _get_debug_mode():
+        block.sanity_check()
+
+
+def _remove_slice_nets(block, skip_sanity_check=False):
+    """ Remove all unneeded slice nodes from the block.
+
+    Unneeded here means that the source and destination wires of a slice net are exactly
+    the same, because the slice takes all the bits, in order, from the source.
+    """
+    # Turns a net of form on the left into the one on the right:
+    #
+    #  w1
+    #   |
+    # [3:0]
+    #   |
+    # [3:0]     ===>  w1
+    #   |             |
+    # [3:0] w2      [3:0] w2
+    #  / \ /         / \ /
+    # ~   +         ~   +
+    # |   |         |   |
+
+    wire_src_dict = _ProducerList()
+    wire_removal_set = set()  # set of all wirevectors to be removed
+
+    def is_net_slicing_entire_wire(net):
+        if net.op != 's':
+            return False
+
+        src_wire = net.args[0]
+        dst_wire = net.dests[0]
+        if len(src_wire) != len(dst_wire):
+            return False
+
+        selLower = net.op_param[0]
+        selUpper = net.op_param[-1]
+        # Check if getting all bits from the src_wire (i.e. consecutive bits, MSB to LSB)
+        return net.op_param == tuple(range(selLower, selUpper + 1))
+
+    # one pass to build the map of value producers and
+    # all of the nets and wires to be removed
+    for net in block.logic:
+        if is_net_slicing_entire_wire(net):
+            wire_src_dict[net.dests[0]] = net.args[0]
+            if not isinstance(net.dests[0], Output):
+                wire_removal_set.add(net.dests[0])
+
+    # second full pass to create the new logic without the wire nets
+    new_logic = set()
+    for net in block.logic:
+        if not is_net_slicing_entire_wire(net) or isinstance(net.dests[0], Output):
+            new_args = tuple(wire_src_dict.find_producer(x) for x in net.args)
+            new_net = LogicNet(net.op, net.op_param, new_args, net.dests)
+            new_logic.add(new_net)
+
     # now update the block with the new logic and remove wirevectors
     block.logic = new_logic
     for dead_wirevector in wire_removal_set:
         del block.wirevector_by_name[dead_wirevector.name]
         block.wirevector_set.remove(dead_wirevector)
 
-    block.sanity_check()
+    if (not skip_sanity_check) or _get_debug_mode():
+        block.sanity_check()
 
 
 def constant_propagation(block, silence_unexpected_net_warnings=False):
@@ -216,8 +283,7 @@ def _constant_prop_pass(block, silence_unexpected_net_warnings=False):
 
 
 def common_subexp_elimination(block=None, abs_thresh=1, percent_thresh=0):
-    """
-    Common Subexpression Elimination for PyRTL blocks
+    """ Common Subexpression Elimination for PyRTL blocks.
 
     :param block: the block to run the subexpression elimination on
     :param abs_thresh: absolute threshold for stopping optimization
@@ -235,6 +301,18 @@ ops_where_arg_order_matters = 'm@xc<>-'
 
 
 def _find_common_subexps(block):
+    """ Finds nets that can be considered the same based on op type, op param, and arguments.
+
+    :param block: Block to operate over
+    :return dict[LogicNet, [LogicNet]]: mapping from a logic net (with a placehold dest)
+        representing the common subexp, to a list of nets matching that common subexp that
+        can be replaced with the single common subexp.
+
+    Nets are the "same" if 1) their op types are the same, 2) their op_params are
+    the same (e.g. same memory if a memory-related op), and 3) their arguments are
+    the same (same constant value and bitwidth for const wires, otherwise same wire
+    object). The destination wire for a net is not considered.
+    """
     net_table = {}  # {net (without dest) : [net, ...]
     t = tuple()  # just a placeholder
     const_dict = {}
@@ -252,6 +330,11 @@ def _find_common_subexps(block):
 
 
 def _const_to_int(wire, const_dict):
+    """ Return a repr a Const (a tuple composed of width and value) for comparison with an 'is'.
+
+    If the wire is not a Const, just return the wire itself; comparison will be
+    done on the identity of the wire object instead.
+    """
     if isinstance(wire, Const):
         # a very bad hack to make sure two consts will compare
         # correctly with an 'is'
@@ -268,6 +351,12 @@ def _const_to_int(wire, const_dict):
 
 
 def _replace_subexps(block, net_table):
+    """ Removes unnecessary nets, connecting the common net's dest wire to unnecessary net's dest.
+
+    :param block: The block to operate over.
+    :param net_table: A mapping from common subexpression (a net) to a list of nets
+        that can be replaced with that common net.
+    """
     wire_map = {}
     unnecessary_nets = []
     for nets in net_table.values():
@@ -282,6 +371,16 @@ def _has_normal_dest_wire(net):
 
 
 def _process_nets_to_discard(nets, wire_map, unnecessary_nets):
+    """ Helper for tracking how a group of related nets should be replaced with a common one.
+
+    :param nets: List of nets that are considered equal and which should
+        be replaced by a single common net.
+    :param wire_map: Dict that will be updated with a mapping from every
+        old destination wire that needs to be removed, to the new destination
+        wire with which it should be replaced.
+    :param unnecessary_nets: List of nets that are to be discarded.
+
+    """
     if len(nets) == 1:
         return  # also deals with nets with no dest wires
     nets_to_consider = list(filter(_has_normal_dest_wire, nets))
@@ -297,7 +396,9 @@ def _process_nets_to_discard(nets, wire_map, unnecessary_nets):
 
 
 def _remove_unlistened_nets(block):
-    """ Removes all nets that are not connected to an output wirevector
+    """ Removes all nets that are not connected to an output wirevector.
+
+    :param block: The block to operate over.
     """
 
     listened_nets = set()
@@ -326,7 +427,12 @@ def _remove_unlistened_nets(block):
 
 
 def _remove_unused_wires(block, keep_inputs=True):
-    """ Removes all unconnected wires from a block"""
+    """ Removes all unconnected wires from a block's wirevector_set.
+
+    :param block: The block to operate over.
+    :param keep_inputs: If True, retain any Input wires that are not connected
+        to any net.
+    """
     valid_wires = set()
     for logic_net in block.logic:
         valid_wires.update(logic_net.args, logic_net.dests)
@@ -344,6 +450,7 @@ def _remove_unused_wires(block, keep_inputs=True):
             PyrtlInternalError("Output wire, " + removed_wire.name + " not driven")
 
     block.wirevector_set = valid_wires
+    block.wirevector_by_name = {wire.name: wire for wire in valid_wires}
 
 # --------------------------------------------------------------------
 #    __           ___       ___  __     __
@@ -352,11 +459,14 @@ def _remove_unused_wires(block, keep_inputs=True):
 #
 
 
-def synthesize(update_working_block=True, block=None):
+def synthesize(update_working_block=True, merge_io_vectors=True, block=None):
     """ Lower the design to just single-bit "and", "or", "xor", and "not" gates.
 
-    :param update_working_block: Boolean specifying if working block update
-    :param block: The block you want to synthesize
+    :param update_working_block: Boolean specifying if working block should
+        be set to the newly synthesized block.
+    :param merge_io_wirevectors: if False, turn all N-bit IO wirevectors
+        into N 1-bit IO wirevectors (i.e. don't maintain interface).
+    :param block: The block you want to synthesize.
     :return: The newly synthesized block (of type PostSynthesisBlock).
 
     Takes as input a block (default to working block) and creates a new
@@ -369,7 +479,10 @@ def synthesize(update_working_block=True, block=None):
     the individual bits and memories (read and write ports) which
     require the reassembly and disassembly of the wirevectors immediately
     before and after. These are the only two places where 'c' and 's' ops
-    should exist.
+    should exist. If merge_io_vectors is False, then these individual
+    bits are not reassembled and disassembled before and after, and so no
+    'c' and 's' ops will exist. Instead, they will be named <name>[n],
+    where n is the bit number of original wire to which it corresponds.
 
     The block that results from synthesis is actually of type
     "PostSynthesisBlock" which contains a mapping from the original inputs
@@ -385,7 +498,9 @@ def synthesize(update_working_block=True, block=None):
 
     block_out = PostSynthBlock()
     # resulting block should only have one of a restricted set of net ops
-    block_out.legal_ops = set('~&|^nrwcsm@')
+    block_out.legal_ops = set('~&|^nrwm@')
+    if merge_io_vectors:
+        block_out.legal_ops.update(set('cs'))
     wirevector_map = {}  # map from (vector,index) -> new_wire
 
     with set_working_block(block_out, no_sanity_check=True):
@@ -400,6 +515,12 @@ def synthesize(update_working_block=True, block=None):
                 ('>', _basic_gt)]:
             net_transform(_replace_op(op, fun), block_in)
 
+        # This is a map from the cloned io wirevector created in copy_block,
+        # to the original io wirevector found in block_pre. We use it to create
+        # the block_out.io_map that is returned to the user.
+        orig_io_map = {temp: orig for orig, temp in block_in.io_map.items()}
+        orig_reg_map = {temp: orig for orig, temp in block_in.reg_map.items()}
+
         # Next, create all of the new wires for the new block
         # from the original wires and store them in the wirevector_map
         # for reference.
@@ -408,23 +529,35 @@ def synthesize(update_working_block=True, block=None):
                 new_name = '_'.join((wirevector.name, 'synth', str(i)))
                 if isinstance(wirevector, Const):
                     new_val = (wirevector.val >> i) & 0x1
-                    new_wirevector = Const(bitwidth=1, val=new_val)
+                    new_wirevector = Const(name=new_name, bitwidth=1, val=new_val)
                 elif isinstance(wirevector, (Input, Output)):
-                    new_wirevector = WireVector(name="tmp_" + new_name, bitwidth=1)
+                    if merge_io_vectors:
+                        new_wirevector = WireVector(name="tmp_" + new_name, bitwidth=1)
+                    else:
+                        # Creating N 1-bit io wires for a given single N-bit io wire.
+                        new_name = wirevector.name
+                        if len(wirevector) > 1:
+                            new_name += '[' + str(i) + ']'
+                        new_wirevector = wirevector.__class__(name=new_name, bitwidth=1)
+                        block_out.io_map[orig_io_map[wirevector]].append(new_wirevector)
                 else:
                     new_wirevector = wirevector.__class__(name=new_name, bitwidth=1)
+                    if isinstance(wirevector, Register):
+                        block_out.reg_map[orig_reg_map[wirevector]].append(new_wirevector)
                 wirevector_map[(wirevector, i)] = new_wirevector
 
         # Now connect up the inputs and outputs to maintain the interface
-        for wirevector in block_in.wirevector_subset(Input):
-            input_vector = Input(name=wirevector.name, bitwidth=len(wirevector))
-            for i in range(len(wirevector)):
-                wirevector_map[(wirevector, i)] <<= input_vector[i]
-        for wirevector in block_in.wirevector_subset(Output):
-            output_vector = Output(name=wirevector.name, bitwidth=len(wirevector))
-            output_bits = [wirevector_map[(wirevector, i)]
-                           for i in range(len(output_vector))]
-            output_vector <<= concat_list(output_bits)
+        if merge_io_vectors:
+            for wirevector in block_in.wirevector_subset(Input):
+                input_vector = Input(name=wirevector.name, bitwidth=len(wirevector))
+                for i in range(len(wirevector)):
+                    wirevector_map[(wirevector, i)] <<= input_vector[i]
+                block_out.io_map[orig_io_map[wirevector]].append(input_vector)
+            for wirevector in block_in.wirevector_subset(Output):
+                output_vector = Output(name=wirevector.name, bitwidth=len(wirevector))
+                output_bits = [wirevector_map[(wirevector, i)] for i in range(len(output_vector))]
+                output_vector <<= concat_list(output_bits)
+                block_out.io_map[orig_io_map[wirevector]].append(output_vector)
 
         # Now that we have all the wires built and mapped, walk all the blocks
         # and map the logic to the equivalent set of primitives in the system
@@ -571,9 +704,7 @@ def and_inverter_synth(net):
 
 @transform.all_nets
 def two_way_concat(net):
-    """
-    Transforms a block such that all n-way (n > 2) concats are replaced
-    with series of 2-way concats.
+    """ Transforms a block so all n-way (n > 2) concats are replaced with series of 2-way concats.
 
     :param block: The block to transform
 
@@ -617,8 +748,7 @@ def two_way_concat(net):
 
 @transform.all_nets
 def one_bit_selects(net):
-    """
-    Converts arbitrary-sliced selects to concatenations of 1-bit selects
+    """ Converts arbitrary-sliced selects to concatenations of 1-bit selects.
 
     :param block: The block to transform
 
@@ -634,3 +764,134 @@ def one_bit_selects(net):
     catlist = [net.args[0][i] for i in net.op_param]
     dest = net.dests[0]
     dest <<= concat_list(catlist)
+
+
+def direct_connect_outputs(block=None):
+    """ Remove 'w' nets immediately before outputs, if possible.
+
+    :param block: block to update (defaults to working block)
+
+    The 'w' nets that are eligible for removal with this pass
+    meet the following requirements:
+        * The destination wirevector of the net is an Output
+        * The source wirevector of the net doesn't go to any other nets.
+    """
+    # Turns a netlist of the form (where [] denote nets and o is an Output):
+    #
+    #  w1 w2
+    #   | |
+    #   [*]
+    #    |
+    #    w3
+    #    |
+    #   [w]
+    #    |
+    #    o
+    #
+    # into:
+    #
+    #  w1 w2
+    #   | |
+    #   [*]
+    #    |
+    #    o
+
+    # NOTE: would use transform.all_nets(), but it becomes tricky when
+    # we want to remove more than just the current net on a single pass
+    block = working_block(block)
+    _, dst_nets = block.net_connections()
+
+    nets_to_remove = set()
+    nets_to_add = set()
+    wirevectors_to_remove = set()
+
+    for net in block.logic:
+        if net.op == '@':
+            continue
+
+        dest_wire = net.dests[0]
+        if dest_wire not in dst_nets or len(dst_nets[dest_wire]) > 1:
+            continue
+
+        dst_net = dst_nets[dest_wire][0]
+        if dst_net.op != 'w' or not isinstance(dst_net.dests[0], Output):
+            continue
+
+        new_net = LogicNet(
+            op=net.op,
+            op_param=net.op_param,
+            args=net.args,
+            dests=dst_net.dests,
+        )
+        nets_to_remove.add(net)
+        nets_to_remove.add(dst_net)
+        wirevectors_to_remove.add(dst_net.args[0])
+        nets_to_add.add(new_net)
+
+    block.logic.difference_update(nets_to_remove)
+    block.logic.update(nets_to_add)
+    for w in wirevectors_to_remove:
+        block.remove_wirevector(w)
+
+
+def _make_tree(wire, block, curr_fanout):
+    def f(w, n):
+        if n == 1:
+            return (w,)
+        else:
+            l_fanout = n // 2
+            r_fanout = n - l_fanout
+            o = WireVector(len(w), block=block)
+            split_net = LogicNet(
+                op='w',
+                op_param=None,
+                args=(w,),
+                dests=(o,),
+            )
+            block.add_net(split_net)
+            return f(o, l_fanout) + f(o, r_fanout)
+    return f(wire, curr_fanout)
+
+
+def two_way_fanout(block=None):
+    """ Update the block such that no wire goes to more than 2 destination nets
+
+    :param block: block to update (defaults to working block)
+    """
+    from .analysis import fanout
+    block = working_block(block)
+
+    _, dst_map = block.net_connections()
+    # Two-pass approach: Remember which nets will need to change, in case
+    # there are multiple arguments which will be changing along the way.
+    nets_to_update = collections.defaultdict(list)
+    for wire in block.wirevector_subset(exclude=(Output)):
+        curr_fanout = fanout(wire)
+        if curr_fanout > 1:
+            s = _make_tree(wire, block, curr_fanout)
+            curr_ix = 0
+            for dst_net in dst_map[wire]:
+                for i, arg in enumerate(dst_net.args):
+                    if arg is wire:
+                        nets_to_update[dst_net].append((wire, i, s[curr_ix]))
+                        curr_ix += 1
+            if curr_ix != curr_fanout:
+                raise PyrtlInternalError("Calculated fanout does not equal number of wires found")
+
+    for old_net, args in nets_to_update.items():
+        def get_arg(i, a):
+            for orig, ix, from_tree in args:
+                # Checking index as well because the same wire could be
+                # used as multiple arguments to the same net.
+                if i == ix and a is orig:
+                    return from_tree
+            return a
+
+        new_net = LogicNet(
+            op=old_net.op,
+            op_param=old_net.op_param,
+            args=tuple(get_arg(ix, a) for ix, a in enumerate(old_net.args)),
+            dests=old_net.dests
+        )
+        block.add_net(new_net)
+        block.logic.remove(old_net)
