@@ -98,15 +98,22 @@ class WireVector(object):
     _code = 'W'
 
     def __init__(self, bitwidth=None, name='', block=None):
-        """ Construct a generic WireVector
+        """ Construct a generic WireVector.
 
         :param int bitwidth: If no bitwidth is provided, it will be set to the
-         minimum number of bits to represent this wire
+            minimum number of bits to represent this wire
         :param Block block: The block under which the wire should be placed.
-         Defaults to the working block
+            Defaults to the working block
         :param String name: The name of the wire referred to in some places.
-         Must be unique. If none is provided, one will be autogenerated
-        :return: a wirevector object representing a const wire
+            Must be unique. If none is provided, one will be autogenerated
+        :return: a wirevector object
+
+        Examples::
+
+            data = pyrtl.WireVector(8, 'data')  # visible in trace as "data"
+            ctrl = pyrtl.WireVector(1)  # assigned tmp name, not visible in traces by default
+            temp = pyrtl.WireVector()  # temporary with width to be defined later
+            temp <<= data  # this this case temp will get the bitwdith of 8 from data
         """
         self._name = None
 
@@ -170,7 +177,14 @@ class WireVector(object):
         return rhs
 
     def __ilshift__(self, other):
-        """ Wire assignment operator (assign other to self). """
+        """ Wire assignment operator (assign other to self).
+
+        Example::
+
+            i = pyrtl.Input(8, 'i')
+            t = pyrtl.WireVector(8, 't')
+            t <<= i
+        """
         other = self._prepare_for_assignment(other)
         self._build(other)
         return self
@@ -226,8 +240,13 @@ class WireVector(object):
     __nonzero__ = __bool__  # for Python 2 and 3 compatibility
 
     def __and__(self, other):
-        """ Creates a LogicNet that ands two wires together into a single wire
-            :return Wirevector: the result wire of the operation
+        """ Creates a LogicNet that ands two wires together into a single wire.
+
+        :return Wirevector: the result wire of the operation
+
+        Example::
+
+                temp = a & b
         """
         return self._two_var_op(other, '&')
 
@@ -238,8 +257,13 @@ class WireVector(object):
         raise PyrtlError('error, operation not allowed on WireVectors')
 
     def __or__(self, other):
-        """ Creates a LogicNet that ors two wires together into a single wire
-            :return Wirevector: the result wire of the operation
+        """ Creates a LogicNet that ors two wires together into a single wire.
+
+        :return Wirevector: the result wire of the operation
+
+            Example::
+
+                temp = a | b
         """
         return self._two_var_op(other, '|')
 
@@ -249,8 +273,9 @@ class WireVector(object):
     # __ior__ used for conditional assignment above
 
     def __xor__(self, other):
-        """ Creates a LogicNet that xors two wires together into a single wire
-            :return Wirevector: the result wire of the operation
+        """ Creates a LogicNet that xors two wires together into a single wire.
+
+        :return Wirevector: the result wire of the operation
         """
         return self._two_var_op(other, '^')
 
@@ -264,9 +289,16 @@ class WireVector(object):
         """ Creates a LogicNet that adds two wires together into a single wirevector.
 
         :return Wirevector: Returns the result wire of the operation.
-          The resulting wire has one more bit than the longer of the two input wires.
+            The resulting wire has one more bit than the longer of the two input wires.
 
         Addition is compatible with two's complement signed numbers.
+
+        Examples::
+
+            temp = a + b  # simple addition of two wirevectors
+            temp = a + 5  # you can use integers
+            temp = a + 0b110  # you can use other integers
+            temp = a + "3'h7"  # compatable verilog constants work too
         """
         return self._two_var_op(other, '+')
 
@@ -280,7 +312,7 @@ class WireVector(object):
         """ Creates a LogicNet that subtracts the right wire from the left one.
 
         :return Wirevector: Returns the result wire of the operation.
-          The resulting wire has one more bit than the longer of the two input wires.
+            The resulting wire has one more bit than the longer of the two input wires.
 
         Subtraction is compatible with two's complement signed numbers.
         """
@@ -298,7 +330,7 @@ class WireVector(object):
         """ Creates a LogicNet that multiplies two different wirevectors.
 
         :return Wirevector: Returns the result wire of the operation.
-          The resulting wire's bitwidth is the sum of the two input wires' bitwidths.
+            The resulting wire's bitwidth is the sum of the two input wires' bitwidths.
 
         Multiplication is *not* compatible with two's complement signed numbers.
         """
@@ -311,46 +343,50 @@ class WireVector(object):
         raise PyrtlError('error, operation not allowed on WireVectors')
 
     def __lt__(self, other):
-        """ Creates a LogicNet that calculates whether a wire is less than another
+        """ Creates a LogicNet that calculates whether a wire is less than another.
 
         :return Wirevector: a one bit result wire of the operation
         """
         return self._two_var_op(other, '<')
 
     def __le__(self, other):
-        """ Creates LogicNets that calculates whether a wire is less than or equal to another
+        """ Creates LogicNets that calculates whether a wire is less than or equal to another.
 
         :return Wirevector: a one bit result wire of the operation
         """
         return ~ self._two_var_op(other, '>')
 
     def __eq__(self, other):
-        """ Creates a LogicNet that calculates whether a wire is equal to another
+        """ Creates a LogicNet that calculates whether a wire is equal to another.
 
         :return Wirevector: a one bit result wire of the operation
         """
         return self._two_var_op(other, '=')
 
     def __ne__(self, other):
-        """ Creates LogicNets that calculates whether a wire not equal to another
+        """ Creates LogicNets that calculates whether a wire not equal to another.
+
         :return Wirevector: a one bit result wire of the operation
         """
         return ~ self._two_var_op(other, '=')
 
     def __gt__(self, other):
-        """ Creates a LogicNet that calculates whether a wire is greater than another
+        """ Creates a LogicNet that calculates whether a wire is greater than another.
+
         :return Wirevector: a one bit result wire of the operation
         """
         return self._two_var_op(other, '>')
 
     def __ge__(self, other):
-        """ Creates LogicNets that calculates whether a wire is greater than or equal to another
+        """ Creates LogicNets that calculates whether a wire is greater than or equal to another.
+
         :return Wirevector: a one bit result wire of the operation
         """
         return ~ self._two_var_op(other, '<')
 
     def __invert__(self):
-        """ Creates LogicNets that inverts a wire
+        """ Creates LogicNets that inverts a wire.
+
         :return Wirevector: a result wire for the operation
         """
         outwire = WireVector(bitwidth=len(self))
@@ -363,7 +399,8 @@ class WireVector(object):
         return outwire
 
     def __getitem__(self, item):
-        """ Grabs a subset of the wires
+        """ Grabs a subset of the wires.
+
         :return Wirevector: a result wire for the operation
         """
         if self.bitwidth is None:
@@ -455,7 +492,7 @@ class WireVector(object):
         """ Generate a new truncated wirevector derived from self.
 
         :return WireVector: Returns a new WireVector equal to
-           the original WireVector but truncated to the specified bitwidth.
+            the original WireVector but truncated to the specified bitwidth.
 
         If the bitwidth specified is larger than the bitwidth of self,
         then PyrtlError is thrown.
@@ -470,7 +507,7 @@ class WireVector(object):
         """ Generate a new sign extended wirevector derived from self.
 
         :return WireVector: Returns a new WireVector equal to
-           the original WireVector sign extended to the specified bitwidth.
+            the original WireVector sign extended to the specified bitwidth.
 
         If the bitwidth specified is smaller than the bitwidth of self,
         then PyrtlError is thrown.
@@ -481,7 +518,7 @@ class WireVector(object):
         """ Generate a new zero extended wirevector derived from self.
 
         :return WireVector: Returns a new WireVector equal to
-           the original WireVector zero extended to the specified bitwidth.
+            the original WireVector zero extended to the specified bitwidth.
 
         If the bitwidth specified is smaller than the bitwidth of self,
         then PyrtlError is thrown.
@@ -509,21 +546,6 @@ class WireVector(object):
             working_block().add_net(net)
             return concat(extvector, self)
 
-    def as_bundle(self, obj):
-        from .helperfuncs import Bundle
-
-        bundle_bw = Bundle.get_bundle_bitwidth(obj)
-
-        if len(self) != bundle_bw:
-            raise PyrtlError(
-                "Width of wire %s (%d) does not equal width of bundle %s (%d)"
-                % (str(self.name), len(self), str(obj.name), bundle_bw)
-            )
-
-        w = Bundle(obj)
-        w <<= self
-        return w
-
 
 # -----------------------------------------------------------------------
 #  ___     ___  ___       __   ___  __           ___  __  ___  __   __   __
@@ -532,7 +554,7 @@ class WireVector(object):
 #
 
 class Input(WireVector):
-    """ A WireVector type denoting inputs to a block (no writers) """
+    """ A WireVector type denoting inputs to a block (no writers). """
     _code = 'I'
 
     def __init__(self, bitwidth=None, name='', block=None):
@@ -556,7 +578,7 @@ class Input(WireVector):
 
 
 class Output(WireVector):
-    """ A WireVector type denoting outputs of a block (no readers)
+    """ A WireVector type denoting outputs of a block (no readers).
 
     Even though Output seems to have valid ops such as __or__ , using
     them will throw an error.
@@ -568,7 +590,7 @@ class Output(WireVector):
 
 
 class Const(WireVector):
-    """ A WireVector representation of a constant value
+    """ A WireVector representation of a constant value.
 
     Converts from bool, integer, or Verilog-style strings to a constant
     of the specified bitwidth.  If the bitwidth is too short to represent
@@ -580,7 +602,7 @@ class Const(WireVector):
     _code = 'C'
 
     def __init__(self, val, bitwidth=None, name='', signed=False, block=None):
-        """ Construct a constant implementation at initialization
+        """ Construct a constant implementation at initialization.
 
         :param int, bool, or str val: the value for the const wirevector
         :param int: the desired bitwidth of the resulting const
@@ -684,9 +706,33 @@ class Register(WireVector):
             self.rhs = rhs
             self.is_conditional = is_conditional
 
-    def __init__(self, bitwidth, name='', block=None):
+    def __init__(self, bitwidth, name='', reset_value=None, block=None):
+        """ Construct a register.
+
+        :param int bitwidth: Number of bits to represent this register.
+        :param String name: The name of the wire.  Must be unique. If none
+            is provided, one will be autogenerated.
+        :param reset_value: Value to initialize this register to during simulation
+            and in any code (e.g. Verilog) that is exported. Defaults to 0, but can
+            be explicitly overridden at simulation time.
+        :param block: The block under which the wire should be placed.
+            Defaults to the working block.
+        :return: a WireVector object representing a register.
+
+        It is an error if the reset_value cannot fit into the specified bitwidth
+        for this register.
+        """
+        from pyrtl.helperfuncs import infer_val_and_bitwidth
+
         super(Register, self).__init__(bitwidth=bitwidth, name=name, block=block)
         self.reg_in = None  # wire vector setting self.next
+        if reset_value is not None:
+            reset_value, rst_bitwidth = infer_val_and_bitwidth(reset_value)
+            if rst_bitwidth > bitwidth:
+                raise PyrtlError(
+                    'reset_value "%s" cannot fit in the specified %d bits for this register'
+                    % (str(reset_value), bitwidth))
+        self.reset_value = reset_value
 
     @property
     def next(self):
